/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @class
 * Defines <code>Mediator</code> names for the application.
 */
var MediatorNames = Objs("org.puremvc.js.demos.objs.employeeadmin.abc.MediatorNames",{});

MediatorNames.USER_FORM_MEDIATOR = "userFormMediator";
MediatorNames.USER_LIST_MEDIATOR = "userListMediator";
/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @class
 * Defines <code>Notification</code> names for the application.
 */
var NotificationNames = Objs("org.puremvc.js.demos.objs.employeeadmin.abc.NotificationNames",{});

NotificationNames.STARTUP = "startup";
NotificationNames.NEW_USER = "newUser";
NotificationNames.DELETE_USER = "deleteUser";
NotificationNames.USER_SELECTED = "userSelected";
NotificationNames.USER_ADDED = "userAdded";
NotificationNames.USER_UPDATED = "userUpdated";
NotificationNames.USER_DELETED = "userDeleted";
NotificationNames.ADD_ROLE = "addRole";
NotificationNames.ADD_ROLE_RESULT = "addRoleResult";
/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @class
 * Defines <code>Proxy</code> names for the application.
 */
var ProxyNames = Objs("org.puremvc.js.demos.objs.employeeadmin.abc.ProxyNames",{});

ProxyNames.ROLE_PROXY = "roleProxy";
ProxyNames.USER_PROXY = "userProxy";
/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @class
 * Command used to delete a user from the main users list.
 *
 * @requires org.puremvc.js.patterns.command.SimpleCommand SimpleCommand
 * @requires org.puremvc.js.patterns.observer.Notification Notification
 * @requires org.puremvc.js.demos.objs.employeeadmin.model.UserProxy UserProxy
 * @requires org.puremvc.js.demos.objs.employeeadmin.model.RoleProxy RoleProxy
 * @requires org.puremvc.js.demos.objs.employeeadmin.model.vo.UserVO UserVO
 * 
 * @extends org.puremvc.js.patterns.command.SimpleCommand SimpleCommand
 *
 * @constructor
 */
var DeleteUserCommand = Objs("org.puremvc.js.demos.objs.employeeadmin.controller.DeleteUserCommand",
	SimpleCommand,
{
	/**
	 * @override
	 */
	execute: function( note )
	{
		var user/*UserVO*/ = note.getBody();
		var userProxy/*UserProxy*/ = this.facade.retrieveProxy( ProxyNames.USER_PROXY );
		var roleProxy/*RoleProxy*/ = this.facade.retrieveProxy( ProxyNames.ROLE_PROXY );

		userProxy.deleteItem( user );
		roleProxy.deleteItem( user );

		this.sendNotification( NotificationNames.USER_DELETED );
	}
});
/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @class
 * Configure and initialize model for the application.
 *
 * @extends org.puremvc.js.patterns.command.SimpleCommand SimpleCommand
 *
 * @constructor
 */
var PrepModelCommand = Objs("org.puremvc.js.demos.objs.employeeadmin.controller.PrepModelCommand",
	SimpleCommand,
{	
	/**
	 * @override
	 */
	execute: function( note )
	{
		this.facade.registerProxy( new UserProxy( ProxyNames.USER_PROXY, this.generateUsers() ) );
		this.facade.registerProxy( new RoleProxy( ProxyNames.ROLE_PROXY , this.generateRoles() ) );
	},
	
	/**
	 * Generates and returns a dummy users list.
	 * 
	 * @return {Array}
	 * 		The generated dummy users list.	
	 */
	generateUsers: function()
	{
		var user/*UserVO*/;
		var users/*Array*/ = new Array();
		
		user = new UserVO();
		user.uname = "lstooge";
		user.fname = "Larry";
		user.lname = "Stooge";
		user.email = "larry@stooges.com";
		user.password = "ijk456";
		user.department = DeptEnum.ACCT;
		users.push(user);
		 
		user = new UserVO();
		user.uname = "cstooge"; 
		user.fname = "Curly";
		user.lname = "Stooge";
		user.email = "curly@stooges.com";
		user.password = "xyz987";
		user.department = DeptEnum.SALES;
		users.push(user);
		
		user = new UserVO();
		user.uname = "mstooge";
		user.fname = "Moe";
		user.lname = "Stooge";
		user.email = "moe@stooges.com";
		user.password = "abc123";
		user.department = DeptEnum.PLANT;
		users.push(user);
		
		return users
	},
	
	/**
	 * Generates and returns a dummy roles list.
	 * 
	 * @return {Array}
	 * 		The generated dummy roles list.	
	 */
	generateRoles: function()
	{
		var role/*RoleVO*/;
		var roles/*Array*/ = new Array();
		
		role = new RoleVO();
		role.uname = "lstooge";
		role.roles = [ RoleEnum.PAYROLL, RoleEnum.EMP_BENEFITS	];
		roles.push(role);
			
		role = new RoleVO();
		role.uname = "cstooge";
		role.roles = [ RoleEnum.ACCT_PAY, RoleEnum.ACCT_RCV, RoleEnum.GEN_LEDGER ];
		roles.push(role);
			
		role = new RoleVO();
		role.uname = "mstooge";
		role.roles = [ RoleEnum.INVENTORY, RoleEnum.PRODUCTION, RoleEnum.SALES, RoleEnum.SHIPPING ];
		roles.push(role);
		
		return roles
	}
});
/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @class
 * Configure and initialize view for the application.
 *
 * @extends org.puremvc.js.patterns.command.SimpleCommand SimpleCommand
 *
 * @constructor
 */
var PrepViewCommand = Objs("org.puremvc.js.demos.objs.employeeadmin.controller.PrepViewCommand",
	SimpleCommand,
{	
	/**
	 * @override
	 */
	execute: function( note )
	{
		/*
		 * View Components initialization
		 */
		var userForm/*UserForm*/ = new UserForm();
		var userList/*UserList*/ = new UserList();

		/*
		 * Mediators initialization
		 */
		var userListMediator/*UserListMediator*/ = new UserListMediator( MediatorNames.USER_LIST_MEDIATOR, userList );
		var userFormMediator/*UserFormMediator*/ = new UserFormMediator( MediatorNames.USER_FORM_MEDIATOR, userForm );

		/*
		 * PureMVC mediators registration
		 */
		this.facade.registerMediator( userFormMediator );
		this.facade.registerMediator( userListMediator );
	}
});
/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @class
 * Start the application.
 *
 * @extends org.puremvc.js.patterns.command.SimpleCommand SimpleCommand
 *
 * @constructor
 */
var StartupCommand = Objs("org.puremvc.js.demos.objs.employeeadmin.controller.StartupCommand",
	MacroCommand,
{
	/**
	 * @override
	 * 
	 * Add the Subcommands to startup the PureMVC apparatus.
	 * 
	 * Generally, it is best to prep the Model (mostly registering 
	 * proxies)followed by preparation of the View (mostly registering 
	 * Mediators).
	 * 
	 * @param {Notification} note
	 * 		The <code>Notification</code> object to be passed to each entry
	 * 		of <i>subCommands</i> list.
	 */
	initializeMacroCommand: function( note )
	{
		this.addSubCommand( PrepModelCommand );
		this.addSubCommand( PrepViewCommand );
	}
});
/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @class
 * An enumeration of department items.
 *
 * @constructor
 */
var DeptEnum = Objs("org.puremvc.js.demos.objs.employeeadmin.model.enum.DeptEnum",
{
	/**
	 * @constructs
	 * Initialize a <code>DeptEnum</code> instance.
	 * 
	 * @param {String} value
	 * 		Value shared by each enum item.
	 * 
	 * @param {Number} ordinal
	 * 		Index of the item in the list.	
	 */
	initialize: function( value, ordinal )
	{
		this.value = value;
		this.ordinal = ordinal;
	},
	
	/**
	 * The <code>DeptEnum</code> entry identifier.
	 * 
	 * @type {Number}
	 */
	ordinal: null,
	
	/**
	 * The <code>DeptEnum</code> entry value.
	 * 
	 * @type {String}
	 */
	value: null,
	
	/**
	 * Compare a <code>DeptEnum</code> object to the current one to check for their
	 * equality.
	 * 
	 * @param {DeptEnum} deptEnum
	 * 		The <code>DeptEnum</code> item to compare to the current.
	 * 
	 * @return {Boolean}
	 * 		The compared <code>DeptEnum</code> is equal to the current.
	 */
	equals: function( deptEnum )
	{
		return ( this.ordinal == deptEnum.ordinal && this.value == deptEnum.value );
	}
});

DeptEnum.NONE_SELECTED/*DeptEnum*/ 	= new DeptEnum( "Select a department", -1 );
DeptEnum.ACCT/*DeptEnum*/ 			= new DeptEnum( "Accounting", 0  );
DeptEnum.SALES/*DeptEnum*/ 			= new DeptEnum( "Sales"	, 1  );
DeptEnum.PLANT/*DeptEnum*/ 			= new DeptEnum( "Plant", 2  );
DeptEnum.SHIPPING/*DeptEnum*/ 		= new DeptEnum( "Shipping", 3  );
DeptEnum.QC/*DeptEnum*/ 			= new DeptEnum( "Quality Control", 4  );

/**
 * Returns the department list excluding the
 * <code>DeptEnum.NONE_SELECTED</code> item used to fill the combo box.
 * 
 * @return {Array}
 * 		The department list excluding the <code>DeptEnum.NONE_SELECTED</code>
 * 		item.
 */
DeptEnum.getList = function()
{
	return [
		DeptEnum.ACCT, 
		DeptEnum.SALES, 
		DeptEnum.PLANT
	];
};

/**
 * Returns the department list including the
 * <code>DeptEnum.NONE_SELECTED</code> item used to fill the combo box.
 * 
 * @return {Array}
 * 		The department list including the <code>DeptEnum.NONE_SELECTED</code>
 * 		item.
 */
DeptEnum.getComboList = function()
{
	var cList/*Array*/ = DeptEnum.getList();
	cList.unshift( DeptEnum.NONE_SELECTED );
	return cList;
};
/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * An enumeration of role items.
 *
 * @constructor
 */
var RoleEnum = Objs("org.puremvc.js.demos.objs.employeeadmin.model.enum.RoleEnum",
{
	/**
	 * The <code>RoleEnum</code> entry identifier.
	 * 
	 * @type {Number}
	 */
	ordinal: null,
	
	
	/**
	 * The <code>RoleEnum</code> entry value.
	 * 
	 * @type {String}
	 */
	value: null,
	
	/**
	 * @constructs
	 * Initialize a <code>RoleEnum</code> instance.
	 * 
	 * @param {String} value
	 * 		Value shared by each enum item.
	 * 
	 * @param {Number} ordinal
	 * 		Index of the item in the list.	
	 */
	initialize: function( value, ordinal )
	{
		this.value = value;
		this.ordinal = ordinal;
	},
	
	/**
	 * Compare a <code>RoleEnum</code> object to the current one to check for their
	 * equality.
	 * 
	 * @param {RoleEnum} roleEnum
	 * 		The <code>RoleEnum</code> item to compare to the current.
	 * 
	 * @return {Boolean}
	 * 		The compared <code>RoleEnum</code> is equal to the current.
	 */
	equals: function( roleEnum/*RoleEnum*/ )
	{
		return ( this.ordinal == roleEnum.ordinal && this.value == roleEnum.value );
	}
});

RoleEnum.NONE_SELECTED/*RoleEnum*/ 	= new RoleEnum( "Select user roles", -1 );
RoleEnum.ADMIN/*RoleEnum*/ 			= new RoleEnum( "Administrator", 0  );
RoleEnum.ACCT_PAY/*RoleEnum*/ 		= new RoleEnum( "Accounts Payable", 1  );
RoleEnum.ACCT_RCV/*RoleEnum*/ 		= new RoleEnum( "Accounts Receivable", 2  );
RoleEnum.EMP_BENEFITS/*RoleEnum*/ 	= new RoleEnum( "Employee Benefits", 3  );
RoleEnum.GEN_LEDGER/*RoleEnum*/ 	= new RoleEnum( "General Ledger", 4  );
RoleEnum.PAYROLL/*RoleEnum*/ 		= new RoleEnum( "Payroll", 5  );
RoleEnum.INVENTORY/*RoleEnum*/ 		= new RoleEnum( "Inventory", 6  );
RoleEnum.PRODUCTION/*RoleEnum*/ 	= new RoleEnum( "Production", 7  );
RoleEnum.QUALITY_CTL/*RoleEnum*/ 	= new RoleEnum( "Quality Control", 8  );
RoleEnum.SALES/*RoleEnum*/ 			= new RoleEnum( "Sales", 9  );
RoleEnum.ORDERS/*RoleEnum*/ 		= new RoleEnum( "Orders",10  );
RoleEnum.CUSTOMERS/*RoleEnum*/ 		= new RoleEnum( "Customers",11  );
RoleEnum.SHIPPING/*RoleEnum*/ 		= new RoleEnum( "Shipping",12  );
RoleEnum.RETURNS/*RoleEnum*/ 		= new RoleEnum( "Returns",13  );


/**
 * Returns the roles list excluding the <code>RoleEnum.NONE_SELECTED</code>
 * item used to fill the combo box.
 * 
 * @return {Array}
 * 		The roles list excluding the <code>RoleEnum.NONE_SELECTED</code>
 * 		item.
 */
RoleEnum.getList = function()
{
	return [
		RoleEnum.ADMIN, 
		RoleEnum.ACCT_PAY, 
		RoleEnum.ACCT_RCV, 
		RoleEnum.EMP_BENEFITS, 
		RoleEnum.GEN_LEDGER, 
		RoleEnum.PAYROLL,
		RoleEnum.INVENTORY,
		RoleEnum.PRODUCTION,
		RoleEnum.QUALITY_CTL,
		RoleEnum.SALES,
		RoleEnum.ORDERS,
		RoleEnum.CUSTOMERS,
		RoleEnum.SHIPPING,
		RoleEnum.RETURNS
	];
};

/**
 * Returns the roles list including the
 * <code>RoleEnum.NONE_SELECTED</code> item used to fill the combo box.
 * 
 * @return {Array}
 * 		The department list including the <code>RoleEnum.NONE_SELECTED</code>
 * 		item.
 */
RoleEnum.getComboList = function()
{
	var cList/*Array*/ = RoleEnum.getList();
	cList.unshift( RoleEnum.NONE_SELECTED );
	return cList;
};

/**
 * Returns the <code>RoleEnum</code> with this ordinal value.
 * 
 * @param {Number} ordinal
 * 		The ordinal value to search for in the list.
 * 
 * @return {RoleEnum}
 * 		The <code>RoleEnum</code> with this ordinal value or <code>null</code>
 * 		if not found.
 */
RoleEnum.getItem = function( ordinal )
{
	var list/*Array*/ = RoleEnum.getList();
	for( var i/*Number*/=0; i<list.length; i++ )
		if( RoleEnum[list[i]].ordinal == ordinal )
			return RoleEnum[list[i]];
	
	return null;
};
/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * The value object in charge of transporting the data to describe each user
 * roles.
 * 
 * @constructor
 */
var RoleVO = Objs("org.puremvc.js.demos.objs.employeeadmin.model.vo.RoleVO",
{
	/**
	 * Unique name of the user to whom is associated the role.
	 * 
	 * @type {String}
	 */
	uname: "",
	
	/**
	 * The list of roles associated to the user.
	 * 
	 * @type {Array}
	 */	
	roles: []
});
/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * The value object in charge of transporting the data to describe each user of
 * the application.
 * 
 * @requires org.puremvc.js.demos.objs.employeeadmin.model.enum.DeptEnum DeptEnum
 * 
 * @constructor
 */
var UserVO = Objs("org.puremvc.js.demos.objs.employeeadmin.model.vo.UserVO",
{
	/** 
	 * Unique name of the user.
	 * 
	 * @type {String}
	 */
	uname: "",
	
	/** 
	 * First name of the user.
	 * 
	 * @type {String}
	 */
	fname: "",
	
	/**  		
	 * Last name of the user.
	 * 
	 * @type {String}
	 */
	lname: "",
	
	/** 		
	 * E-mail name of the user.
	 * 
	 * @type {String}
	 */
	email: "",
	
	/** 		
	 * Password name of the user.
	 * 
	 * @type {String}
	 */
	password: "",
	
	/**		
	 * The <code>DeptEnum</code> item associated to the user.
	 * 
	 * @type {DeptEnum}
	 */
	department: DeptEnum.NONE_SELECTED,
	
	/**
	 * Indicate if the data shared by the value object are valid.
	 * 
	 * @return {Boolean}
	 * 		The data shared by the value object are valid.
	 */
	getIsValid: function()
	{
		return 	this.uname != "" 
				&&
				this.password != ""
				&&
				this.department != DeptEnum.NONE_SELECTED
		;
	},
	
	/**
	 * Return the complete name for this user.
	 * 
	 * @return {Boolean}
	 * 		The complete name for this user.
	 */
	getGivenName: function()
	{
		return this.lname + ", " + this.fname;
	}
});
/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * PureMVC <code>Proxy</code> class object used to control the user roles list
 * of the application. 
 * 
 * @requires org.puremvc.js.patterns.proxy.Proxy Proxy
 * @requires org.puremvc.js.patterns.observer.Notification Notification
 * @requires org.puremvc.js.demos.objs.employeeadmin.model.vo.RoleVO RoleVO
 * @requires org.puremvc.js.demos.objs.employeeadmin.model.vo.UserVO UserVO
 * @requires org.puremvc.js.demos.objs.employeeadmin.model.enum.RoleEnum RoleEnum
 * @requires org.puremvc.js.demos.objs.employeeadmin.ApplicationFacade ApplicationFacade
 * 
 * @extends org.puremvc.js.patterns.proxy.Proxy Proxy
 * 
 * @constructor
 */
var RoleProxy = Objs("org.puremvc.js.demos.objs.employeeadmin.model.RoleProxy",
	Proxy,
{

	/**
	 * @constructs
	 * @override
	 *
	 * Initialize a <code>RoleProxy</code> instance.
	 *
	 * @param {String} name
	 * 		Identifier of the <code>Proxy</code> object in the PureMVC framework.
	 *
	 * @param {Array} roles
	 * 		The list of user roles <code>RoleVO</code> object controlled by the
	 * 		<code>Proxy</code>.
	 */
	initialize: function( name, roles )
	{
		RoleProxy.$super.initialize.call( this, name, roles );

		return this;
	},

	/**
	 * Get the role list.
	 *
	 * @return {Array}
	 * 		The role list.
	 */
	getRoles: function()
	{
		return this.data;
	},

	/**
	 * Add a role to the list.
	 *
	 * @param {RoleVO} role
	 * 		The role to add.
	 */
	addItem: function( role )
	{
		this.getRoles().push( role );
	},

	/**
	 * Remove a role from the list.
	 *
	 * @param {RoleVO} item
	 * 		The role to remove.
	 */
	deleteItem: function( item )
	{
		var roles/*Array*/ = this.getRoles();
		for( var i/*Number*/=0; i<roles.length; i++ )
		{
			if( roles[i].uname == item.uname )
			{
				roles.splice(i,1);
				break;
			}
		}
	},

	/**
	 * Determine if the user has a given role.
	 *
	 * @param {UserVO} user
	 * 		The user for whom to search for the role.
	 *
	 * @param {RoleEnum} role
	 * 		The role to search for.
	 *
	 * @return {Boolean}
	 * 		The user has the given role.
	 */
	doesUserHaveRole: function( user, role )
	{
		var roles/*Array*/ = this.getRoles();
		var hasRole/*Boolean*/ = false;
		for( var i/*Number*/=0; i<roles.length; i++)
		{
			if( roles[i].uname == user.uname )
			{
				var userRoles/*Array*/ = roles[i].roles;
				for( var j/*Number*/=0; j<userRoles.length; j++ )
				{
					var roleEnum/*RoleEnum*/ = userRoles[j];
					if( roleEnum.equals( role ) )
					{
						hasRole = true;
						break;
					}
				}
				break;
			}
		}
		return hasRole;
	},

	/**
	 * Add a role to a user.
	 *
	 * @param {UserVO} user
	 * 		The user to whom to add a role.
	 *
	 * @param {RoleEnum} role
	 * 		The role to add.
	 */
	addRoleToUser: function( user, role )
	{
		var roles/*Array*/ = this.getRoles();
		var result/*Boolean*/ = false;
		if ( !this.doesUserHaveRole(user, role) )
		{
			for( var i/*Number*/=0; i<roles.length; i++)
			{
				if( roles[i].uname == user.uname )
				{
					var userRoles/*Array*/ = roles[i].roles;
					userRoles.push( role );
					result = true;
					break;
				}
			}
		}
	},

	/**
	 * Remove a role from a user.
	 *
	 * @param {UserVO} user
	 * 		The user to whom remove the role.
	 *
	 * @param {RoleEnum} role
	 * 		The role to remove.
	 */
	removeRoleFromUser: function( user, role )
	{
		var roles/*Array*/ = this.getRoles();
		if( this.doesUserHaveRole( user, role ) )
		{
			for( var i/*Number*/=0; i<roles.length; i++)
			{
				if( roles[i].uname == user.uname )
				{
					var userRoles/*Array*/ = roles[i].roles;
					for( var j/*Number*/=0; j<userRoles.length; j++)
					{
						var roleEnum/*RoleEnum*/ = userRoles[j];
						if( roleEnum.equals( role ) )
						{
							userRoles.splice(j,1);
							break;
						}
					}
					break;
				}
			}
		}
	},

	/**
	 * Get a user's roles.
	 *
	 * @param {String} uname
	 * 		The user unique name.
	 *
	 * @return {Array}
	 * 		The user's role list.
	 */
	getUserRoles: function( uname )
	{
		var roles/*Array*/ = this.getRoles();
		var userRoles/*Array*/ = new Array();
		for( var i/*Number*/=0; i<roles.length; i++)
		{
			if( roles[i].uname == uname )
			{
				userRoles = roles[i].roles;
				break;
			}
		}

		return userRoles;
	}
});
/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @classDescription
 * PureMVC <code>Proxy</code> class object used to control the user list of the
 * application. 
 * 
 * @requires org.puremvc.js.patterns.proxy.Proxy Proxy
 * @requires org.puremvc.js.demos.objs.employeeadmin.abc.ProxyNames ProxyNames
 * @requires org.puremvc.js.demos.objs.employeeadmin.model.vo.UserVO UserVO
 * @requires org.puremvc.js.demos.objs.employeeadmin.model.enum.DeptEnum DeptEnum
 * 
 * @extends org.puremvc.js.patterns.proxy.Proxy Proxy
 * 
 * @constructor
 */
var UserProxy = Objs("org.puremvc.js.demos.objs.employeeadmin.model.UserProxy",
	Proxy,
{
	/**
	 * @constructs
	 * @override
	 *
	 * Initialize a <code>UserProxy</code> instance.
	 *
	 * @param {String} name
	 * 		Identifier of the <code>Proxy</code> object in the PureMVC framework.
	 *
	 * @param {Array} users
	 * 		The list of users controlled by the <code>Proxy</code>.
	 */
	initialize: function( name, users )
	{
		UserProxy.$super.initialize.call( this, name, users );
	},

	/**
	 * Return the users list controlled by the <code>Proxy</code>.
	 */
	getUsers: function()
	{
		return this.data;
	},

	/**
	 * Add a user to the list.
	 *
	 * @param {UserVO} user
	 */
	addItem: function( user )
	{
		this.getUsers().push( user );
	},

	/**
	 * Return a user given its user name.
	 *
	 * @param {String} uname
	 * 		The user name of the user to find.
	 *
	 * @return {UserVO}
	 * 		The user with the given user name or null if none exists with
	 * 		this user name.
	 */
	getUser: function( uname )
	{
		var users/*Array*/ = this.getUsers();
		for( var i/*Number*/=0; i<users.length; i++ )
			if( users[i].uname == uname )
				return users[i];

		return null;
	},

	/**
	 * Update a user informations.
	 *
	 * @param {UserVO} user
	 * 		The user to update.
	 */
	updateItem: function( user )
	{
		var users/*Array*/ = this.getUsers();
		for( var i/*Number*/=0; i<users.length; i++ )
			if( users[i].uname == user.uname )
				users[i] = user;
	},

	/**
	 * Remove a user from the list.
	 *
	 * @param {UserVO} user
	 * 		The user to remove.
	 */
	deleteItem: function( user )
	{
		var users/*Array*/ = this.getUsers();
		for( var i/*Number*/=0; i<users.length; i++ )
			if( users[i].uname == user.uname )
				users.splice(i,1);
	}
});
/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @class
 * A base class used for UI components of the application.
 * 
 * <P>
 * It mainly adds a basic UiComponent implementation to make UI components
 * listenable from the <code>Mediator</code>s. Here to simplify the demo we
 * don't use a real <code>Event</code> class. Implementers and listeners are
 * responsible for the anonymous events object they dispatch and receive.
 *
 * @constructor
 */
var UiComponent = Objs("org.puremvc.js.demos.objs.employeeadmin.view.components.UiComponent",
{
	/**
	 * @constructs
	 * Initialize a <code>UiComponent</code> instance.
	 */
	initialize: function()
	{
		this.listenerMap = {};
	},
	
	/**
	 * A map of <code>UiComponent.listenerDescriptor</code> objects.
	 * 
	 * @type {Object}
	 * @private
	 */
	listenerMap: null,
	
	/**
	* Dispatches an event into the event flow.
	* 
	* @param {String} type
	* 		The type of the event to dispatch.
	* 
	* @param {Object} properties
	* 		An optional anonymous object to send to listeners of the event when it
	* 		is dispatched.
	*/
	dispatchEvent: function( type, properties )
	{
		if( typeof type == 'undefined' )
			return;
			
		if( typeof this.listenerMap[UiComponent.QUEUE_PATTERN + type] == 'undefined' )
			return;
	
		var queue/*Array*/ = this.listenerMap[UiComponent.QUEUE_PATTERN + type].slice(0);
		
		var props/*Object*/ = properties || {};
		var len/*Number*/ = queue.length;
		for(var i/*Number*/=0; i<len; i++)
		{
			var listenerDescriptor/*UiComponent.ListenerDescriptor*/ = queue[i];
	
			if( typeof listenerDescriptor.listener == 'function' )
			{
				if( typeof listenerDescriptor.context != "undefined" )
					listenerDescriptor.listener.call( listenerDescriptor.context, props );
				else
					listenerDescriptor.listener.call( this, event, props );
			}
		}
	},
	
	/**
	* Add an event listener so that the listener receives notification of an event.
	 *  
	 * @param {String} type
	 * 		Type of the event to add.
	 * 
	 * @param {Function} listener
	 * 		The listener method of the event to add.
	 * 
	 * @param {Object} context
	 * 		The context attached for the listener method of the event to remove.
	 */
	addEventListener: function
	(
		type,
		listener,
		context
	)
	{
		if( typeof type == "undefined" )
			return;
	
		if( typeof listener == "undefined" )
			return;
			
		var newListener/*UiComponent.ListenerDescriptor*/ = new UiComponent.ListenerDescriptor( listener, context );
	
		var queue/*Object*/;
		if( typeof this.listenerMap[ UiComponent.QUEUE_PATTERN + type ] == "undefined" )
			queue = this.listenerMap[ UiComponent.QUEUE_PATTERN + type ] = [];
		else
			queue = this.listenerMap[ UiComponent.QUEUE_PATTERN + type ];
	
		var len/*Number*/ = queue.length;
		for(var i/*Number*/=0; i<len; i++ )
		{
			var listenerDescriptor/*UiComponent.ListenerDescriptor*/ = queue[i];
			if( listenerDescriptor.equals( newListener ) )
				return;
		}
	
		queue.push(newListener);
	},
	
	/**
	 * Remove an event listener so that the listener stops receiving notification
	 * of an event.
	 *  
	 * @param {String} type
	 * 		Type of the event to remove.
	 * 
	 * @param {Function} listener
	 * 		The listener method of the event to remove.
	 * 
	 * @param {Object} context
	 * 		The context attached for the listener method of the event to remove.
	 */
	removeEventListener: function
	(
		type,
		listener,
		context
	)
	{
		if( typeof type == "undefined" )
			return;
	
		if( typeof listener == "undefined" )
			return;
	
		if( typeof this.listenerMap[UiComponent.QUEUE_PATTERN + type] == "undefined" )
			return;
			
		var queue/*Object*/ = this.listenerMap[ UiComponent.QUEUE_PATTERN + type ];
		var len/*Number*/ = queue.length;
		for(var i/*Number*/=0; i<len; i++)
        {
            var listenerDescriptor/*UiComponent.ListenerDescriptor*/ = queue[i];
            if( listenerDescriptor.equals( new UiComponent.ListenerDescriptor( listener, context ) ) )
			{
				queue.splice(i,1);
				return;
			}
        }
	}
});

/**
 * @class
 * @private
 * The event object dispatched by the <code>UiComponent</code> class to its
 * event listeners.
 */
UiComponent.Event = Objs("org.puremvc.js.demos.objs.employeeadmin.view.components.UiComponent.Event",
{
	/**
	 * Type of the dispatched event.
	 * 
	 * @type {String}
	 */
	type: null,
	
	/**
	 * Properties that follow the dispatched event.
	 * 
	 * @type {Object}	
	 */
	properties: null

});

/**
 * @private
 * 
 * A descriptor object used by the <code>UiComponent.listenerMap</code>
 * to identify each event listener.
 * 
 * <P>
 * It is intentionally not declared on prototype as it built a kind of inner
 * class for JavaScript.
 */
UiComponent.ListenerDescriptor = Objs("org.puremvc.js.demos.objs.employeeadmin.view.components.UiComponent.Event",
{
	/**
	 * @construct
	 * Initialize a <code>UiComponent.ListenerDescriptor</code> instance.
	 * 
	 * @param {Function} listener
	 * 		The listener method to call.
	 * 
	 * @param {Function} listener
	 * 		The listener context on which to call the method.
	 */	
	initialize: function( listener, context )
	{
		this.listener = listener;
		this.context = context;
	},

    /**
     * @private
     *
     * Compare two <code>UiComponent.ListenerDescriptor</code>s to determine if
     * they target the exact same event listener.
     *
     * @param {UiComponent.ListenerDescriptor} compared
     * 		The descriptor that will be compared to the current.
     *
     * @return {Boolean}
     * 		The two compared listeners are equals.
     */
    equals: function( compared )
    {
        if( compared.listener == this.listener )
        {
            if( typeof compared.context != "undefined" )
            {
                if( compared.context == null && this.context == null )
                    return true;

                if( compared.context == this.context )
                    return true;
            }
        }

        return false;
    }
});

/* 
 * Private statics
 */
	
/**
 * @private
 * 
 * A prefix used on map item names to prevent name conflicts.
 * 
 * @type {String}
 * @constant
 */
UiComponent.QUEUE_PATTERN = '@_@';
/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @class
 * The UI component in charge of the <em>user form</em>.
 * 
 * @extends org.puremvc.js.demos.objs.employeeadmin.view.components.UiComponent UiComponent
 *
 * @constructor
 */
var UserForm = Objs("org.puremvc.js.demos.objs.employeeadmin.view.components.UserForm",
	UiComponent,
{
	/**
	 * The user form panel HTML element.
	 *
	 * @private
	 * @type {HTMLElement}
	 */
	userFormPanel: null,

	/**
	 * The unique name field HTML element.
	 *
	 * @private
	 * @type {HTMLElement}
	 */
	uname: null,

	/**
	 * The first name field HTML element.
	 *
	 * @private
	 * @type {HTMLElement}
	 */
	fname: null,

	/**
	 * The long name field HTML element.
	 *
	 * @private
	 * @type {HTMLElement}
	 */
	lname: null,

	/**
	 * The email field HTML element.
	 *
	 * @private
	 * @type {HTMLElement}
	 */
	email: null,

	/**
	 * The password field HTML element.
	 *
	 * @private
	 * @type {HTMLElement}
	 */
	password: null,

	/**
	 * The confirm password field HTML element.
	 *
	 * @private
	 * @type {HTMLElement}
	 */
	confirm: null,

	/**
	 * The department field HTML element.
	 *
	 * @private
	 * @type {HTMLElement}
	 */
	department: null,

	/**
	 * The submit button HTML element.
	 *
	 * @private
	 * @type {HTMLElement}
	 */
	submitButton: null,

	/**
	 * The selected user.
	 *
	 * @private
	 * @type {UserVO}
	 */
	user: null,

	/**
	 * The roles list for the selected user.
	 *
	 * @private
	 * @type {Array}
	 */
	userRoles: null,

	/**
	 * @private
	 * @type {String}
	 */
	mode: null,

	/**
	 * @constructs
	 * @override
	 *
	 * Initialize a <code>UserForm</code> instance.
	 */
	initialize: function()
	{
		UserForm.$super.initialize.call( this );

		this.initializeChildren();
		this.bindListeners();
	},

	/**
	 * Initialize references to DOM elements.
	 */
	initializeChildren: function()
	{
		/*
		 * We use JQuery to initialize reference to UI components
		 */
		this.userFormPanel = jQuery("#user-form-panel");

		this.uname = this.userFormPanel.find("#uname");
		this.fname = this.userFormPanel.find("#fname");
		this.lname = this.userFormPanel.find("#lname");
		this.email = this.userFormPanel.find("#email");
		this.password = this.userFormPanel.find("#password");
		this.confirm = this.userFormPanel.find("#confirm");
		this.department = this.userFormPanel.find("#department").selectmenu();
		this.roles = this.userFormPanel.find("#roles").selectmenu();

		this.submitButton = this.userFormPanel.find("#submit-button").button();
		this.deleteButton = this.userFormPanel.find("#delete-button").button();
	},

	/**
	 * Bind events to their listeners.
	 */
	bindListeners: function()
	{
		var focusEventProxy/*jQueryProxy*/ = jQuery.proxy( this, "field_focusHandler" );
		this.uname.on("focus" + UserForm.EVENT_NS, focusEventProxy );
		this.password.on("focus" + UserForm.EVENT_NS, focusEventProxy );
		this.confirm.on("focus" + UserForm.EVENT_NS, focusEventProxy );

		this.submitButton.on( "click" + UserForm.EVENT_NS, jQuery.proxy( this, "submitButton_clickHandler" ) );
		this.deleteButton.on( "click" + UserForm.EVENT_NS, jQuery.proxy( this, "deleteButton_clickHandler" ) );
	},

	/**
	 * Unbind events from their listeners.
	 */
	unbindListeners: function()
	{
		this.uname.off("focus" + UserForm.EVENT_NS );
		this.password.off("focus" + UserForm.EVENT_NS );
		this.confirm.off("focus" + UserForm.EVENT_NS );

		this.department.parent().off("click" + UserForm.EVENT_NS );
		this.roles.parent().off("click" + UserForm.EVENT_NS );

		this.submitButton.off( "click" + UserForm.EVENT_NS );
		this.deleteButton.off( "click" + UserForm.EVENT_NS );
	},

	/**
	 * Add items from <code>DeptEnum</code> to the corresponding list UI
	 * component.
	 *
	 * @param {Array} deptEnumList
	 *		List of <code>DeptEnum</code> items or an empty array to empty
	 *		the list UI component content.
	 */
	fillDepartmentList: function( deptEnumList )
	{
		/*
		 * Whatever the documentation says, with jQuery Mobile 1.0.1 we need to
		 * hard reset the select component for jQuery Mobile to refresh its
		 * display and set which is its selected element.
		 */
		var select/*String*/ = '<select id="department"><option></option></select>';
		var options/*String*/ = "";
		for( var i/*Number*/=0; i<deptEnumList.length; i++ )
		{
			var deptEnum/*DeptEnum*/ = deptEnumList[i];

			var valueAttr = 'value="' + deptEnum.ordinal + '"';
			var selectedAttr/*String*/ = deptEnum.equals(this.user.department) ? 'selected' : "";
			options += '<option ' + valueAttr + ' ' + selectedAttr + ' >' + deptEnum.value + '</option>';
		}

		if( options )
			select = select.replace("<option></option>", options);

		//It's all I found to "cleanly" reset the component.
		var container = this.department.closest(".ui-field-contain");
		var parent = this.department.closest(".ui-select");
		parent.detach();
		container.append(select);

		this.department = this.userFormPanel.find("#department").selectmenu();
		this.department.selectmenu('refresh');

		this.department.off("change" + UserForm.EVENT_NS );
		this.department.on("change" + UserForm.EVENT_NS, jQuery.proxy( this, "field_focusHandler" ) );
},

	/**
	 * Add items from <code>RoleEnum</code> to the corresponding list UI component.
	 *
	 * @param {Array} roleEnumList
	 *		List of <code>RoleEnum</code> items or an empty array to empty
	 *		the list UI component content.
	 */
	fillRoleList: function( roleEnumList )
	{
		/*
		 * Whatever the documentation says, with jQuery Mobile 1.0.1 we need to
		 * hard reset the select component for jQuery Mobile to refresh its
		 * display and set which is its selected element.
		 */
		var select/*String*/ = '<select id="roles" multiple="multiple"><option></option></select>';
		var options/*String*/ = "";
		for( var i/*Number*/=0; i<roleEnumList.length; i++ )
		{
			var roleEnum/*RoleEnum*/ = roleEnumList[i];

			var valueAttr/*String*/ = "";
			if( !roleEnum.equals(RoleEnum.NONE_SELECTED ) )
				valueAttr = 'value="' + roleEnum.ordinal + '"';

			var selectedAttr/*String*/ = this.isUserRole( roleEnum ) ? 'selected' : "";
			options += '<option ' + valueAttr + ' ' + selectedAttr + ' >' + roleEnum.value + '</option>\n';
		}

		if( options )
			select = select.replace("<option></option>", options);

		//It's all I found to "cleanly" reset the component.
		var container = this.roles.closest(".ui-field-contain");
		var parent = this.roles.closest(".ui-select");
		parent.detach();
		container.append(select);

		this.roles = this.userFormPanel.find("#roles").selectmenu();
		this.roles.selectmenu("refresh");

		this.roles.off("change" + UserForm.EVENT_NS );
		this.roles.on("change" + UserForm.EVENT_NS, jQuery.proxy( this, "field_focusHandler" ) );
	},

	/**
	 * Set the user used to populate the form.
	 *
	 * @param {UserVO} user
	 * 		The currently selected user.
	 */
	setUser: function( user )
	{
		this.user = user;

		this.uname.val(user.uname);
		this.fname.val(user.fname);
		this.lname.val(user.lname);
		this.email.val(user.email);
		this.password.val(user.password);
		this.confirm.val(user.password);

		this.fillDepartmentList( DeptEnum.getComboList() );
	},

	getUser: function()/*UserVO*/
	{
		var user/*UserVO*/ = new UserVO();
		user.uname = this.uname.val();
		user.fname = this.fname.val();
		user.lname = this.lname.val();
		user.email = this.email.val();
		user.password = this.password.val();
		user.department = this.getSelectedDepartment();

		return user;
	},

	/**
	 * Set the roles for the selected user.
	 *
	 * @param {Array} userRoles
	 * 		The roles list for the currently selected user.
	 */
	setUserRoles: function( userRoles )
	{
		this.userRoles = userRoles;
		this.fillRoleList( RoleEnum.getComboList() );
	},

	/**
	 * Return the roles selected in the UI role list component for the user.
	 *
	 * @return {Array}
	 * 		The list of <code>RoleEnum</code> object selected for the user.
	 */
	getUserRoles: function()
	{
		return this.getSelectedRoles();
	},

	/**
	 * Clear the whole form.
	 */
	clearForm: function()
	{
		this.uname.val("");
		this.fname.val("");
		this.lname.val("");
		this.email.val("");
		this.password.val("");
		this.confirm.val("");
		this.fillRoleList( [] );
		this.fillDepartmentList( [] );
		this.setFieldError( "uname", false );
		this.setFieldError( "password", false );
		this.setFieldError( "confirm", false );
		this.setFieldError( "department", false );

		this.fillDepartmentList([]);
		this.fillRoleList([]);
	},

	/**
	 * Set the form mode to ADD or EDIT.
	 *
	 * @param {String} mode
	 * 		<code>UserForm.MODE_ADD</code> or <code>UserForm.MODE_EDIT</code>
	 */
	setMode: function( mode )
	{
		this.mode = mode;

		switch(mode)
		{
			case UserForm.MODE_ADD:
				this.uname.removeAttr("disabled");

				//We have to hack jQuery Mobile a little here
				this.submitButton.parent().find(".ui-btn-text").text("Add");
				this.deleteButton.parent().hide();
			break;

			case UserForm.MODE_EDIT:
				this.uname.attr( "disabled", "disabled" );

				//We have to hack jQuery Mobile a little here
				this.submitButton.parent().find(".ui-btn-text").text("Save");
				this.deleteButton.parent().show();
			break;
		}
	},

	/**
	 * Submit the add or update.
	 */
	submitButton_clickHandler: function()
	{
		if( this.getErrors() )
			return;

		var user/*UserVO*/ = this.getUser();
		if( user.getIsValid() )
		{
			if( this.mode == UserForm.MODE_ADD )
				this.dispatchEvent( UserForm.ADD_USER );
			else
				this.dispatchEvent( UserForm.UPDATE_USER );

			jQuery.mobile.changePage("#UserList");
		}
	},

	/**
	 * Remove button onclick event listener.
	 */
	deleteButton_clickHandler: function()
	{
		this.dispatchEvent( UserForm.DELETE_USER );

		jQuery.mobile.changePage("#UserList");
	},

	/**
	 * Handle focus event on all the required form fields.
	 */
	field_focusHandler: function( evt )
	{
		//Remove error on the selected field.
		this.setFieldError( evt.target.id, false );
	},

	/**
	 * Display errors associated with form fields and return if at least one
	 * field is in error.
	 *
	 * @return {Boolean}
	 * 		The form contains errors.
	 */
	getErrors: function()
	{
		var error/*Boolean*/ = false;
		if( this.uname.val() == "" )
			this.setFieldError( "uname", error = true );
		else
			this.setFieldError( "uname", false );

		if( this.password.val() == "" )
			this.setFieldError( "password", error = true );
		else
			this.setFieldError( "password", false );

		if( this.password.val() != "" && this.confirm.val() != this.password.val() )
			this.setFieldError( "confirm", error = true );
		else
			this.setFieldError( "confirm", false );

		var department/*DeptEnum*/ = this.getSelectedDepartment();
		if( !department || department.equals(DeptEnum.NONE_SELECTED) )
			this.setFieldError( "department", error = true );
		else
			this.setFieldError( "department", false );

		var selectedRoles/*Array*/ = this.getUserRoles();
		if( !selectedRoles || selectedRoles.length == 0 )
			this.setFieldError( "roles", error = true );
		else
			this.setFieldError( "roles", false );

		return error;
	},

	/**
	 * Set or unset the error state on the uname field.
	 *
	 * @param {String} fieldName
	 * 		Name of the field to mark as (or not mark as) containing an error.
	 *
	 * @param {Boolean} error
	 * 		The field must be marked as containing an error.
	 */
	setFieldError: function( fieldName, error )
	{
		var label/*HTMLElement*/ = this.userFormPanel.find( 'label[for="' + fieldName + '"]' );
		var field/*HTMLElement*/ = this.userFormPanel.find( "#" + fieldName );

		if( error )
			field.closest(".ui-field-contain").addClass( "fieldError" );
		else
			field.closest(".ui-field-contain").removeClass( "fieldError" );
	},

	/**
	 * Helper method that checks if a role from the UI role list exists in the user
	 * roles list.
	 *
	 * @param {RoleEnum} roleEnum
	 * 		The <code>RoleEnum</code> item to check for	existence in the user roles
	 * 		list.
	 *
	 * @return {Boolean}
	 * 		The role exists in the list.
	 *
	 * @private
	 */
	isUserRole: function( roleEnum )
	{
	 	for( var i/*Number*/=0; i<this.userRoles.length; i++ )
			if( roleEnum.equals(this.userRoles[i]) )
				return true;

		return false;
	},

	/**
	 * Helper method that returns currently selected department from the
	 * department combobox.
	 *
	 * @return {DeptEnum}
	 * 		Currently selected departement or null.
	 *
	 * @private
	 */
	getSelectedDepartment: function()
	{
		var selectedValue/*Number*/ = parseInt(this.department.val());
		var deptEnumList/*Array*/ = DeptEnum.getList();

		for( var i/*Number*/=0; i<deptEnumList.length; i++ )
		{
			var department/*DeptEnum*/ = deptEnumList[i];
			if( deptEnumList[i].ordinal == selectedValue )
				return department;
		}

		return null;
	},

	/**
	 * Helper method that returns currently selected user roles.
	 *
	 * @return {Array}
	 * 		A list representing the user roles.
	 *
	 * @private
	 */
	getSelectedRoles: function()
	{
		var arr/*Array*/ = [];

		var selectedIndexes/*Array*/ = this.roles.val();
		if( !selectedIndexes )
			return arr;

		var roleEnumList/*Array*/ = RoleEnum.getList();
		for( var i/*Number*/=0; i<roleEnumList.length; i++ )
		{
			var roleEnum/*RoleEnum*/ = roleEnumList[i];
			for(var j/*Number*/=0; j<selectedIndexes.length; j++)
			{
				var selectedValue/*Number*/ = parseInt(selectedIndexes[j]);
				if( roleEnum.ordinal == selectedValue )
					arr.push( roleEnum );
			}
		}

		return arr;
	}
});

/**
 * @const
 * @private
 */
UserForm.EVENT_NS		= ".UserForm";

/*
 * Event names
 */
UserForm.ADD_USER		= "addUser";
UserForm.UPDATE_USER	= "updateUser";
UserForm.DELETE_USER	= "deleteUser";
UserForm.CANCEL			= "cancel";

UserForm.MODE_ADD		= "modeAdd";
UserForm.MODE_EDIT		= "modeEdit";

UserForm.ADD_ROLE		= "addRole";
UserForm.REMOVE_ROLE	= "removeRole";
/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @class
 * The UI component in charge of the <em>User List</em>.
 * 
 * @extends org.puremvc.js.demos.objs.employeeadmin.view.components.UiComponent UiComponent
 *
 * @constructor
 */
var UserList = Objs("org.puremvc.js.demos.objs.employeeadmin.view.components.UserList",
	UiComponent,
{

	/**
	 * The user list panel HTML element.
	 * 
	 * @type {HTMLElement}
	 * @private
	 */
	userListPanel: null,
	
	/**
	 * The user list HTML element.
	 * 
	 * @type {HTMLElement}
	 * @private
	 */
	userList: null,
	
	/**
	 * The "new" button HTML element.
	 * 
	 * @type {HTMLElement}
	 * @private
	 */
	newButton: null,

	/**
	 * The current selected user.
	 * 
	 * @type {String}
	 * @private
	 */
	selectedUser: null,
	
	/**
	 * The user list of the application.
	 * 
	 * @type {Array}
	 * @private
	 */
	users: null,

	/**
	 * @type {jQueryProxy}
	 * @private
	 */
	listItemClickProxy: null,

	/**
	 * @constructs
	 * @override
	 * Initialize a <code>UserList</code> instance.
	 */
	initialize: function()
	{
		UserList.$super.initialize.call( this );

		this.listItemClickProxy = jQuery.proxy( this, "userList_clickHandler" );

		this.initializeChildren();
		this.bindListeners();
	},
	
	/**
     * Initialize references to DOM elements.
     */
    initializeChildren: function()
    {
		/*
		 * We use JQuery to initialize reference to UI components
		 */
		this.userListPanel = jQuery("#user-list-panel");

		this.userList = this.userListPanel.find("#user-list");
		this.newButton = this.userListPanel.find("#new-button").button();
    },

	/**
	 * Bind events to their listeners.
	 */
	bindListeners: function()
	{
		//jQuery will be able to only remove events attached under this namespace
		var namespace/*String*/ = ".UserList";
		this.newButton.on( "click"+namespace, jQuery.proxy( this, "newButton_clickHandler" ) );
	},

	/**
	 * Unbind events from their listeners.
	 */
	unbindListeners: function()
	{
		//jQuery will only remove events attached under this namespace
		var namespace/*String*/ = ".UserList";
		this.newButton.off( "click"+namespace );
	},

	/**
	 * Add users from a list to the <SELECT> component.
	 *
	 * @param {Array} userList
	 * 		The user list to set.
	 */
	setUsers: function( userList )
	{
		this.users = userList;

		var htmlList/*String*/ = "";
		for( var i/*Number*/ = 0; i<this.users.length; i++ )
		{
			var user/*UserVO*/ = this.users[i];
			htmlList += '<li id="' + i  + '-user-list-item"><a href="#">' + user.getGivenName() + '</a></li>';
		}

		/*
		 * Please see http://jquerymobile.com/test/docs/lists/docs-lists.html
		 * for more explanation on why we do this.
		 */
		this.userList.html(htmlList).listview('refresh', true);

		//Here is some jQuery magic
		this.userList.find("li").on( "click", this.listItemClickProxy );
	},

	/**
	 * List selection click event listener.
	 *
	 * <P>
	 * We are currently unable to listen for a <code>select</code> event on an
	 * unumbered list in jQuery Mobile so we have to listen for the click on the
	 * whole list and identify the clicked <li /> element.
	 *
	 * @param {Event} event
	 * 		The native Event propagated by jQuery.
	 */
	userList_clickHandler: function( event )
	{
		var li/*HTMLElement*/ = jQuery(event.currentTarget);

		//Index of the clicked list item
		var entry/*Number*/ = parseInt( li.attr("id") );
		jQuery.mobile.changePage("#UserForm");
		this.dispatchEvent( UserList.SELECT, this.users[entry] );
	},

	/**
	 * New button click event listener.
	 */
	newButton_clickHandler: function()
	{
		jQuery.mobile.changePage("#UserForm");
		this.dispatchEvent( UserList.NEW );
	}

});

/*
 * Events type
 */
UserList.NEW		= "new";
UserList.DELETE 	= "delete";
UserList.SELECT 	= "select";
/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @class
 * User form component <code>Mediator</code>.
 *
 * @requires org.puremvc.js.patterns.mediator.Mediator Mediator
 * @requires org.puremvc.js.patterns.observer.Notification Notification
 * @requires org.puremvc.js.demos.objs.employeeadmin.model.vo.UserVO UserVO
 * @requires org.puremvc.js.demos.objs.employeeadmin.model.UserProxy UserProxy
 * @requires org.puremvc.js.demos.objs.employeeadmin.model.enum.DeptEnum DeptEnum
 * @requires org.puremvc.js.demos.objs.employeeadmin.view.components.UserForm UserForm
 *
 * @extends org.puremvc.js.patterns.mediator.Mediator Mediator
 *
 * @constructor
 */
var UserFormMediator = Objs("org.puremvc.js.demos.objs.employeeadmin.view.UserFormMediator",
	Mediator,
{	

	/**
	 * @private
	 *
	 * A shortcut to the application <code>RoleProxy</code> instance.
	 * 
	 * @type {RoleProxy}
	 */
	roleProxy: null,
		
	/**
	 * @private
	 *
	 * A shortcut to the application <code>UserProxy</code> instance.
	 * 
	 * @type {UserProxy}
	 */
	userProxy: null,

	/**
	 * @constructs
	 * @override
	 *
	 * Initialize a <code>UserFormMediator</code> instance.
	 * 
	 * @param {String} name
	 * 		Name for this <code>Mediator</code>.
	 *
	 * @param {UserForm} viewComponent
	 * 		The <code>UserForm</code> view Component this <code>Mediator</code>
	 * 		manage.
	 */
	initialize: function( name, viewComponent )
	{
		UserFormMediator.$super.initialize.call( this, name, viewComponent );
	
		this.registerListeners();

		this.userProxy = this.facade.retrieveProxy( ProxyNames.USER_PROXY );
		this.roleProxy = this.facade.retrieveProxy( ProxyNames.ROLE_PROXY );
	},
			
	/**
	 * @private
	 * 
	 * The <code>UserForm</code> view component this <code>Mediator</code> manage.
	 * 
	 * @return {UserForm}
	 */
	getUserForm : function()
	{
		return this.viewComponent;
	},

	/**
	 * Register event listeners for the UserForm component.
	 */
	registerListeners: function()
	{
		var userForm/*UserForm*/ = this.getUserForm();
		userForm.addEventListener( UserForm.ADD_USER, this.onAdd, this );
		userForm.addEventListener( UserForm.UPDATE_USER, this.onUpdate, this );
		userForm.addEventListener( UserForm.DELETE_USER, this.onDelete, this );
	},

	/**
	 * Unregister event listeners for the UserForm component.
	 */
	unregisterListeners: function()
	{
		var userForm/*UserForm*/ = this.getUserForm();
		userForm.removeEventListener( UserForm.ADD_USER, this.onAdd, this );
		userForm.removeEventListener( UserForm.UPDATE_USER, this.onUpdate, this );
		userForm.removeEventListener( UserForm.DELETE_USER, this.onDelete, this );
	},

	/**
	 * @private
	 * 
	 * Called when a user is added using the form.
	 * 
	 * @param {UiComponent.Event} event
	 * 		The dispatched event object.
	 */
	onAdd: function( event )
	{
		var user/*UserVO*/ = this.getUserForm().getUser();
		this.userProxy.addItem( user );
		this.updateUserRoles();
		this.sendNotification( NotificationNames.USER_ADDED, user );
	},
		
	/**
	 * @private
	 * 
	 * Called when a user is updated using the form.
	 */
	onUpdate: function()
	{
		var user/*UserVO*/ = this.getUserForm().getUser();
		this.userProxy.updateItem( user );
		this.updateUserRoles();
		this.sendNotification(  NotificationNames.USER_UPDATED, user );

	},
		
	/**
	 * @private
	 * 
	 * Called when a user is deleted.
	 */
	onDelete: function()
	{
		var user/*UserVO*/ = this.getUserForm().getUser();
		if( user == null )
			return;
		
		this.sendNotification
		(
			NotificationNames.DELETE_USER,
			user
		);
	},

	/*
	 * In the jQuery Mobile EmployeeAdmin demo we can't add/remove roles
	 * one by one, so we refresh the whole list on add/update operations.
	 */
	updateUserRoles: function()
	{
		var user/*UserVO*/ = this.getUserForm().getUser();
		var userRoles/*Array*/ = this.getUserForm().getUserRoles();
		var roleVO/*RoleVO*/ = new RoleVO();
		roleVO.uname = user.uname;
		roleVO.roles = userRoles;
		this.roleProxy.deleteItem( roleVO );
		this.roleProxy.addItem( roleVO );
	},
		
	/**
	 * @override
	 */
	listNotificationInterests: function()
	{
		return [
			NotificationNames.NEW_USER,
			NotificationNames.USER_SELECTED
		];
	},
		
	/**
	 * @override
	 */
	handleNotification: function( note )
	{
		var userForm/*UserForm*/ = this.getUserForm();
			
		var user/*UserVO*/;
		switch ( note.getName() )
		{
			case NotificationNames.NEW_USER:
				userForm.clearForm();
				userForm.setUser( note.getBody() );
				userForm.setUserRoles( [] );
				userForm.setMode( UserForm.MODE_ADD );
			break;
		
			case NotificationNames.USER_SELECTED:
				user = note.getBody();
		
				userForm.clearForm();
				userForm.setUser( user );
		
				var roles/*Array*/ = this.roleProxy.getUserRoles(user.uname);
				userForm.setUserRoles( roles );
		
				userForm.setMode( UserForm.MODE_EDIT );
			break;
		}
	},

	/**
	 * @override
	 *
	 * This will never be called during the demo but note that we well made the
	 * job of removing any listeners from the mediator and the component to
	 * make those instances ready for garbage collection.
	 */
	onRemove: function()
	{
		this.unregisterListeners();
		this.getUserForm().unbindListeners();
	}
});

/*
 * Constants
 */
UserFormMediator.ADD			= "add";
UserFormMediator.UPDATE			= "update";
UserFormMediator.CANCEL			= "cancel";

UserFormMediator.MODE_ADD		= "modeAdd";
UserFormMediator.MODE_EDIT		= "modeEdit";
/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @class
 * User list component <code>Mediator</code>.
 * 
 * @requires org.puremvc.js.patterns.mediator.Mediator Mediator
 * @requires org.puremvc.js.patterns.observer.Notification Notification
 * @requires org.puremvc.js.demos.objs.employeeadmin.model.vo.UserVO UserVO
 * @requires org.puremvc.js.demos.objs.employeeadmin.model.UserProxy UserProxy
 * @requires org.puremvc.js.demos.objs.employeeadmin.view.components.UserList UserList
 *
 * @extends org.puremvc.js.patterns.mediator.Mediator Mediator
 */
var UserListMediator = Objs("org.puremvc.js.demos.objs.employeeadmin.view.UserListMediator",
	Mediator,
{

	/**
	 * The <code>UserList</code> UI component this <code>Mediator</code>
	 * manage.
	 *
	 * @type {UserList}
	 */
	userList: null,
	
	/**
	 * @constructs
	 * @override
	 *
	 * Initialize a <code>UserListMediator</code> instance.
	 * 
	 * @param {String} name
	 * 		Name for this <code>Mediator</code>.
	 *
	 * @param {UserList} viewComponent
	 * 		The <code>UserList</code> UI Component this <code>Mediator</code>
	 * 		manage.
	 */
	initialize: function( name, viewComponent )
	{
		UserListMediator.$super.initialize.call( this, name, viewComponent );

		this.registerListeners();

		var userProxy/*UserProxy*/ = this.facade.retrieveProxy( ProxyNames.USER_PROXY );
		viewComponent.setUsers(userProxy.getUsers());
	},
	
	/**
	 * Register event listeners for the UserForm component.
	 */
	registerListeners: function()
	{
		var userList/*UserList*/ = this.getUserList();
		userList.addEventListener( UserList.NEW, this.onNew, this );
		userList.addEventListener( UserList.SELECT, this.onSelect, this );
	},

	/**
	 * Unregister event listeners for the UserForm component.
	 */
	unregisterListeners: function()
	{
		var userList/*UserList*/ = this.getUserList();
		userList.removeEventListener( UserList.NEW, this.onNew, this );
		userList.removeEventListener( UserList.SELECT, this.onSelect, this );
	},
	
	/**
	 * @private
	 * 
	 * Return the <code>UserList</code> UI component this
	 * <code>Mediator</code> manage.
	 * 
	 * @return {UserList}
	 * 		The <code>UserList</code> UI component this
	 * 		<code>Mediator</code> manage.
	 */
	getUserList: function()
	{
		return this.viewComponent;
	},
	
	/**
	 * @override
	 */
	listNotificationInterests: function()
	{
		return [
			NotificationNames.USER_UPDATED,
			NotificationNames.USER_ADDED,
			NotificationNames.USER_DELETED
		];
	},

	/**
	 * @override
	 */
	handleNotification: function( note )
	{
		var userList/*UserList*/ = this.getUserList();
		var userProxy/*UserProxy*/ = this.facade.retrieveProxy( ProxyNames.USER_PROXY );

		switch( note.getName() )
		{
			case NotificationNames.USER_UPDATED:
				userList.setUsers( userProxy.getUsers() );
			break;

			case NotificationNames.USER_ADDED:
				userList.setUsers( userProxy.getUsers() );
			break;

			case NotificationNames.USER_DELETED:
				userList.setUsers( userProxy.getUsers() );
			break;
		}
	},
	
	/**
	 * Called when to add a new user to the list.
	 * 
	 * @private
	 */
	onNew: function()
	{
		var user/*UserVO*/ = new UserVO();
		this.sendNotification( NotificationNames.NEW_USER, user );
	},


	/**
	 * @private
	 *
	 * Called when a user is selected in the user list.
	 *
	 * @param {UserVO} selectedUser
	 * 		The user selected in the list.
	 */
	onSelect: function( selectedUser )
	{
		this.sendNotification( NotificationNames.USER_SELECTED, selectedUser );
	},

	/**
	 * @override
	 *
	 * This will never be called during the demo but note that we well made the
	 * job of removing any listeners from the mediator and the component to
	 * make those instances ready for garbage collection.
	 */
	onRemove: function()
	{
		this.unregisterListeners();
		this.getUserList().unbindListeners();
	}

});
/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @class
 * PureMVC <code>Facade</code> for this application.
 * 
 * @requires org.puremvc.js.patterns.facade.Facade Facade
 * @requires org.puremvc.js.patterns.observer.Notification Notification
 * @requires org.puremvc.js.demos.objs.employeeadmin.controller.StartupCommand StartupCommand
 * @requires org.puremvc.js.demos.objs.employeeadmin.controller.DeleteUserCommand DeleteUserCommand
 * @requires org.puremvc.js.demos.objs.employeeadmin.controller.AddRoleResultCommand AddRoleResultCommand
 * 
 * @extends org.puremvc.js.patterns.facade.Facade Facade
 */
var ApplicationFacade = Objs("org.puremvc.js.demos.objs.employeeadmin.ApplicationFacade",
	Facade,
{
		
	/**
	 * Start the application
	 * 
	 * @param {HTMLElement} app
	 * 		The HTML root node element of the application.
	 */
	startup: function( app )
	{
		this.sendNotification( NotificationNames.STARTUP, app );
	},
		
	/**
	 * The <code>Model</code> <code>View</code> and
	 * <code>Controller</code> are initialized in a deliberate
	 * order to ensure internal dependencies are satisfied before
	 * operations are performed.
	 * 
	 * <P>
	 * <code>initializeController()</code> should be overridden
	 * for the specific purpose of registering your commands. Any attempt to
	 * register <code>Mediator</code>s here will result in an error.
	 * being thrown because the View has not yet been initialized.</p>
	 * <p>calling <code>this.parent()</code> is also required.
	 * 
	 * @override
	 */
	initializeController: function()
	{
		ApplicationFacade.$super.initializeController.call( this );
		
		this.registerCommand( NotificationNames.STARTUP, StartupCommand );
		this.registerCommand( NotificationNames.DELETE_USER, DeleteUserCommand );
	}
});

/**
 * Singleton implementation for the <code>ApplicationFacade</code>.
 *
 * @return {ApplicationFacade}
 * 		The <code>Facade</code> subclass instance used throughout the
 * 		application.
 */
ApplicationFacade.getInstance = function()
{
	if( !Facade.instance ) 
		Facade.instance = new ApplicationFacade();
	
	return Facade.instance;
};
