/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @class
 * The UI component in charge of the <em>user form</em>.
 * 
 * @extends org.puremvc.js.demos.objs.employeeadmin.view.components.UiComponent UiComponent
 *
 * @constructor
 */
var UserForm = Objs("org.puremvc.js.demos.objs.employeeadmin.view.components.UserForm",
	UiComponent,
{
	/**
	 * The user form panel HTML element.
	 *
	 * @private
	 * @type {HTMLElement}
	 */
	userFormPanel: null,

	/**
	 * The unique name field HTML element.
	 *
	 * @private
	 * @type {HTMLElement}
	 */
	uname: null,

	/**
	 * The first name field HTML element.
	 *
	 * @private
	 * @type {HTMLElement}
	 */
	fname: null,

	/**
	 * The long name field HTML element.
	 *
	 * @private
	 * @type {HTMLElement}
	 */
	lname: null,

	/**
	 * The email field HTML element.
	 *
	 * @private
	 * @type {HTMLElement}
	 */
	email: null,

	/**
	 * The password field HTML element.
	 *
	 * @private
	 * @type {HTMLElement}
	 */
	password: null,

	/**
	 * The confirm password field HTML element.
	 *
	 * @private
	 * @type {HTMLElement}
	 */
	confirm: null,

	/**
	 * The department field HTML element.
	 *
	 * @private
	 * @type {HTMLElement}
	 */
	department: null,

	/**
	 * The submit button HTML element.
	 *
	 * @private
	 * @type {HTMLElement}
	 */
	submitButton: null,

	/**
	 * The selected user.
	 *
	 * @private
	 * @type {UserVO}
	 */
	user: null,

	/**
	 * The roles list for the selected user.
	 *
	 * @private
	 * @type {Array}
	 */
	userRoles: null,

	/**
	 * @private
	 * @type {String}
	 */
	mode: null,

	/**
	 * @constructs
	 * @override
	 *
	 * Initialize a <code>UserForm</code> instance.
	 */
	initialize: function()
	{
		UserForm.$super.initialize.call( this );

		this.initializeChildren();
		this.bindListeners();
	},

	/**
	 * Initialize references to DOM elements.
	 */
	initializeChildren: function()
	{
		/*
		 * We use JQuery to initialize reference to UI components
		 */
		this.userFormPanel = jQuery("#user-form-panel");

		this.uname = this.userFormPanel.find("#uname");
		this.fname = this.userFormPanel.find("#fname");
		this.lname = this.userFormPanel.find("#lname");
		this.email = this.userFormPanel.find("#email");
		this.password = this.userFormPanel.find("#password");
		this.confirm = this.userFormPanel.find("#confirm");
		this.department = this.userFormPanel.find("#department").selectmenu();
		this.roles = this.userFormPanel.find("#roles").selectmenu();

		this.submitButton = this.userFormPanel.find("#submit-button").button();
		this.deleteButton = this.userFormPanel.find("#delete-button").button();
	},

	/**
	 * Bind events to their listeners.
	 */
	bindListeners: function()
	{
		var focusEventProxy/*jQueryProxy*/ = jQuery.proxy( this, "field_focusHandler" );
		this.uname.on("focus" + UserForm.EVENT_NS, focusEventProxy );
		this.password.on("focus" + UserForm.EVENT_NS, focusEventProxy );
		this.confirm.on("focus" + UserForm.EVENT_NS, focusEventProxy );

		this.submitButton.on( "click" + UserForm.EVENT_NS, jQuery.proxy( this, "submitButton_clickHandler" ) );
		this.deleteButton.on( "click" + UserForm.EVENT_NS, jQuery.proxy( this, "deleteButton_clickHandler" ) );
	},

	/**
	 * Unbind events from their listeners.
	 */
	unbindListeners: function()
	{
		this.uname.off("focus" + UserForm.EVENT_NS );
		this.password.off("focus" + UserForm.EVENT_NS );
		this.confirm.off("focus" + UserForm.EVENT_NS );

		this.department.off("change" + UserForm.EVENT_NS );
		this.roles.off("change" + UserForm.EVENT_NS );

		this.submitButton.off( "click" + UserForm.EVENT_NS );
		this.deleteButton.off( "click" + UserForm.EVENT_NS );
	},

	/**
	 * Add items from <code>DeptEnum</code> to the corresponding list UI
	 * component.
	 *
	 * @param {Array} deptEnumList
	 *		List of <code>DeptEnum</code> items or an empty array to empty
	 *		the list UI component content.
	 */
	fillDepartmentList: function( deptEnumList )
	{
		/*
		 * Whatever the documentation says, with jQuery Mobile 1.0.1 we need to
		 * hard reset the select component for jQuery Mobile to refresh its
		 * display and set which is its selected element.
		 */
		var select/*String*/ = '<select id="department"><option></option></select>';
		var options/*String*/ = "";
		for( var i/*Number*/=0; i<deptEnumList.length; i++ )
		{
			var deptEnum/*DeptEnum*/ = deptEnumList[i];

			var valueAttr = 'value="' + deptEnum.ordinal + '"';
			var selectedAttr/*String*/ = deptEnum.equals(this.user.department) ? 'selected' : "";
			options += '<option ' + valueAttr + ' ' + selectedAttr + ' >' + deptEnum.value + '</option>';
		}

		if( options )
			select = select.replace("<option></option>", options);

		//It's all I found to "cleanly" reset the component.
		var container = this.department.closest(".ui-field-contain");
		var parent = this.department.closest(".ui-select");
		parent.detach();
		container.append(select);

		this.department = this.userFormPanel.find("#department").selectmenu();
		this.department.selectmenu('refresh');

		//Focus event is not dispatched by the jQuery Mobile select menus
		this.department.off("change" + UserForm.EVENT_NS );
		this.department.on("change" + UserForm.EVENT_NS, jQuery.proxy( this, "field_focusHandler" ) );
},

	/**
	 * Add items from <code>RoleEnum</code> to the corresponding list UI component.
	 *
	 * @param {Array} roleEnumList
	 *		List of <code>RoleEnum</code> items or an empty array to empty
	 *		the list UI component content.
	 */
	fillRoleList: function( roleEnumList )
	{
		/*
		 * Whatever the documentation says, with jQuery Mobile 1.0.1 we need to
		 * hard reset the select component for jQuery Mobile to refresh its
		 * display and set which is its selected element.
		 */
		var select/*String*/ = '<select id="roles" multiple="multiple"><option></option></select>';
		var options/*String*/ = "";
		for( var i/*Number*/=0; i<roleEnumList.length; i++ )
		{
			var roleEnum/*RoleEnum*/ = roleEnumList[i];

			var valueAttr/*String*/ = "";
			if( !roleEnum.equals(RoleEnum.NONE_SELECTED ) )
				valueAttr = 'value="' + roleEnum.ordinal + '"';

			var selectedAttr/*String*/ = this.isUserRole( roleEnum ) ? 'selected' : "";
			options += '<option ' + valueAttr + ' ' + selectedAttr + ' >' + roleEnum.value + '</option>\n';
		}

		if( options )
			select = select.replace("<option></option>", options);

		//It's all I found to "cleanly" reset the component.
		var container = this.roles.closest(".ui-field-contain");
		var parent = this.roles.closest(".ui-select");
		parent.detach();
		container.append(select);

		this.roles = this.userFormPanel.find("#roles").selectmenu();
		this.roles.selectmenu("refresh");

		//Focus event is not dispatched by the jQuery Mobile select menus
		this.roles.off("change" + UserForm.EVENT_NS );
		this.roles.on("change" + UserForm.EVENT_NS, jQuery.proxy( this, "field_focusHandler" ) );
	},

	/**
	 * Set the user used to populate the form.
	 *
	 * @param {UserVO} user
	 * 		The currently selected user.
	 */
	setUser: function( user )
	{
		this.user = user;

		this.uname.val(user.uname);
		this.fname.val(user.fname);
		this.lname.val(user.lname);
		this.email.val(user.email);
		this.password.val(user.password);
		this.confirm.val(user.password);

		this.fillDepartmentList( DeptEnum.getComboList() );
	},

	getUser: function()/*UserVO*/
	{
		var user/*UserVO*/ = new UserVO();
		user.uname = this.uname.val();
		user.fname = this.fname.val();
		user.lname = this.lname.val();
		user.email = this.email.val();
		user.password = this.password.val();
		user.department = this.getSelectedDepartment();

		return user;
	},

	/**
	 * Set the roles for the selected user.
	 *
	 * @param {Array} userRoles
	 * 		The roles list for the currently selected user.
	 */
	setUserRoles: function( userRoles )
	{
		this.userRoles = userRoles;
		this.fillRoleList( RoleEnum.getComboList() );
	},

	/**
	 * Return the roles selected in the UI role list component for the user.
	 *
	 * @return {Array}
	 * 		The list of <code>RoleEnum</code> object selected for the user.
	 */
	getUserRoles: function()
	{
		return this.getSelectedRoles();
	},

	/**
	 * Clear the whole form.
	 */
	clearForm: function()
	{
		this.uname.val("");
		this.fname.val("");
		this.lname.val("");
		this.email.val("");
		this.password.val("");
		this.confirm.val("");
		this.fillRoleList( [] );
		this.fillDepartmentList( [] );
		this.setFieldError( "uname", false );
		this.setFieldError( "password", false );
		this.setFieldError( "confirm", false );
		this.setFieldError( "department", false );

		this.fillDepartmentList([]);
		this.fillRoleList([]);
	},

	/**
	 * Set the form mode to ADD or EDIT.
	 *
	 * @param {String} mode
	 * 		<code>UserForm.MODE_ADD</code> or <code>UserForm.MODE_EDIT</code>
	 */
	setMode: function( mode )
	{
		this.mode = mode;

		switch(mode)
		{
			case UserForm.MODE_ADD:
				this.uname.removeAttr("disabled");

				//We have to hack jQuery Mobile a little here
				this.submitButton.parent().find(".ui-btn-text").text("Add");
				this.deleteButton.parent().hide();
			break;

			case UserForm.MODE_EDIT:
				this.uname.attr( "disabled", "disabled" );

				//We have to hack jQuery Mobile a little here
				this.submitButton.parent().find(".ui-btn-text").text("Save");
				this.deleteButton.parent().show();
			break;
		}
	},

	/**
	 * Submit the add or update.
	 */
	submitButton_clickHandler: function()
	{
		if( this.getErrors() )
			return;

		var user/*UserVO*/ = this.getUser();
		if( user.getIsValid() )
		{
			if( this.mode == UserForm.MODE_ADD )
				this.dispatchEvent( UserForm.ADD_USER );
			else
				this.dispatchEvent( UserForm.UPDATE_USER );

			jQuery.mobile.changePage("#UserList");
		}
	},

	/**
	 * Remove button onclick event listener.
	 */
	deleteButton_clickHandler: function()
	{
		this.dispatchEvent( UserForm.DELETE_USER );

		jQuery.mobile.changePage("#UserList");
	},

	/**
	 * Handle focus event on all the required form fields.
	 */
	field_focusHandler: function( evt )
	{
		//Remove error on the selected field.
		this.setFieldError( evt.target.id, false );
	},

	/**
	 * Display errors associated with form fields and return if at least one
	 * field is in error.
	 *
	 * @return {Boolean}
	 * 		The form contains errors.
	 */
	getErrors: function()
	{
		var error/*Boolean*/ = false;
		if( this.uname.val() == "" )
			this.setFieldError( "uname", error = true );
		else
			this.setFieldError( "uname", false );

		if( this.password.val() == "" )
			this.setFieldError( "password", error = true );
		else
			this.setFieldError( "password", false );

		if( this.password.val() != "" && this.confirm.val() != this.password.val() )
			this.setFieldError( "confirm", error = true );
		else
			this.setFieldError( "confirm", false );

		var department/*DeptEnum*/ = this.getSelectedDepartment();
		if( !department || department.equals(DeptEnum.NONE_SELECTED) )
			this.setFieldError( "department", error = true );
		else
			this.setFieldError( "department", false );

		var selectedRoles/*Array*/ = this.getUserRoles();
		if( !selectedRoles || selectedRoles.length == 0 )
			this.setFieldError( "roles", error = true );
		else
			this.setFieldError( "roles", false );

		return error;
	},

	/**
	 * Set or unset the error state on the uname field.
	 *
	 * @param {String} fieldName
	 * 		Name of the field to mark as (or not mark as) containing an error.
	 *
	 * @param {Boolean} error
	 * 		The field must be marked as containing an error.
	 */
	setFieldError: function( fieldName, error )
	{
		var label/*HTMLElement*/ = this.userFormPanel.find( 'label[for="' + fieldName + '"]' );
		var field/*HTMLElement*/ = this.userFormPanel.find( "#" + fieldName );

		if( error )
			field.closest(".ui-field-contain").addClass( "fieldError" );
		else
			field.closest(".ui-field-contain").removeClass( "fieldError" );
	},

	/**
	 * Helper method that checks if a role from the UI role list exists in the user
	 * roles list.
	 *
	 * @param {RoleEnum} roleEnum
	 * 		The <code>RoleEnum</code> item to check for	existence in the user roles
	 * 		list.
	 *
	 * @return {Boolean}
	 * 		The role exists in the list.
	 *
	 * @private
	 */
	isUserRole: function( roleEnum )
	{
	 	for( var i/*Number*/=0; i<this.userRoles.length; i++ )
			if( roleEnum.equals(this.userRoles[i]) )
				return true;

		return false;
	},

	/**
	 * Helper method that returns currently selected department from the
	 * department combobox.
	 *
	 * @return {DeptEnum}
	 * 		Currently selected departement or null.
	 *
	 * @private
	 */
	getSelectedDepartment: function()
	{
		var selectedValue/*Number*/ = parseInt(this.department.val());
		var deptEnumList/*Array*/ = DeptEnum.getList();

		for( var i/*Number*/=0; i<deptEnumList.length; i++ )
		{
			var department/*DeptEnum*/ = deptEnumList[i];
			if( deptEnumList[i].ordinal == selectedValue )
				return department;
		}

		return null;
	},

	/**
	 * Helper method that returns currently selected user roles.
	 *
	 * @return {Array}
	 * 		A list representing the user roles.
	 *
	 * @private
	 */
	getSelectedRoles: function()
	{
		var arr/*Array*/ = [];

		var selectedIndexes/*Array*/ = this.roles.val();
		if( !selectedIndexes )
			return arr;

		var roleEnumList/*Array*/ = RoleEnum.getList();
		for( var i/*Number*/=0; i<roleEnumList.length; i++ )
		{
			var roleEnum/*RoleEnum*/ = roleEnumList[i];
			for(var j/*Number*/=0; j<selectedIndexes.length; j++)
			{
				var selectedValue/*Number*/ = parseInt(selectedIndexes[j]);
				if( roleEnum.ordinal == selectedValue )
					arr.push( roleEnum );
			}
		}

		return arr;
	}
});

/**
 * @const
 * @private
 */
UserForm.EVENT_NS		= ".UserForm";

/*
 * Event names
 */
UserForm.ADD_USER		= "addUser";
UserForm.UPDATE_USER	= "updateUser";
UserForm.DELETE_USER	= "deleteUser";
UserForm.CANCEL			= "cancel";

UserForm.MODE_ADD		= "modeAdd";
UserForm.MODE_EDIT		= "modeEdit";

UserForm.ADD_ROLE		= "addRole";
UserForm.REMOVE_ROLE	= "removeRole";