/*
 PureMVC Javascript Objs Employee Admin Demo for jQuery Mobile
 by Frederic Saunier <frederic.saunier@puremvc.org> 
 PureMVC - Copyright(c) 2006-12 Futurescale, Inc., Some rights reserved.
 Your reuse is governed by the Creative Commons Attribution 3.0 License
*/

/**
 * @class
 * The UI component in charge of the <em>User List</em>.
 * 
 * @extends org.puremvc.js.demos.objs.employeeadmin.view.components.UiComponent UiComponent
 *
 * @constructor
 */
var UserList = Objs("org.puremvc.js.demos.objs.employeeadmin.view.components.UserList",
	UiComponent,
{

	/**
	 * The user list panel HTML element.
	 * 
	 * @type {HTMLElement}
	 * @private
	 */
	userListPanel: null,
	
	/**
	 * The user list HTML element.
	 * 
	 * @type {HTMLElement}
	 * @private
	 */
	userList: null,
	
	/**
	 * The "new" button HTML element.
	 * 
	 * @type {HTMLElement}
	 * @private
	 */
	newButton: null,

	/**
	 * The current selected user.
	 * 
	 * @type {String}
	 * @private
	 */
	selectedUser: null,
	
	/**
	 * The user list of the application.
	 * 
	 * @type {Array}
	 * @private
	 */
	users: null,

	/**
	 * @type {jQueryProxy}
	 * @private
	 */
	listItemClickProxy: null,

	/**
	 * @constructs
	 * @override
	 * Initialize a <code>UserList</code> instance.
	 */
	initialize: function()
	{
		UserList.$super.initialize.call( this );

		this.listItemClickProxy = jQuery.proxy( this, "userList_clickHandler" );

		this.initializeChildren();
		this.bindListeners();
	},
	
	/**
     * Initialize references to DOM elements.
     */
    initializeChildren: function()
    {
		/*
		 * We use JQuery to initialize reference to UI components
		 */
		this.userListPanel = jQuery("#user-list-panel");

		this.userList = this.userListPanel.find("#user-list");
		this.newButton = this.userListPanel.find("#new-button").button();
    },

	/**
	 * Bind events to their listeners.
	 */
	bindListeners: function()
	{
		//jQuery will be able to only remove events attached under this namespace
		var namespace/*String*/ = ".UserList";
		this.newButton.on( "click"+namespace, jQuery.proxy( this, "newButton_clickHandler" ) );
	},

	/**
	 * Unbind events from their listeners.
	 */
	unbindListeners: function()
	{
		//jQuery will only remove events attached under this namespace
		var namespace/*String*/ = ".UserList";
		this.newButton.off( "click"+namespace );
	},

	/**
	 * Add users from a list to the <SELECT> component.
	 *
	 * @param {Array} userList
	 * 		The user list to set.
	 */
	setUsers: function( userList )
	{
		this.users = userList;

		var htmlList/*String*/ = "";
		for( var i/*Number*/ = 0; i<this.users.length; i++ )
		{
			var user/*UserVO*/ = this.users[i];
			htmlList += '<li id="' + i  + '-user-list-item"><a href="#">' + user.getGivenName() + '</a></li>';
		}

		/*
		 * Please see http://jquerymobile.com/test/docs/lists/docs-lists.html
		 * for more explanation on why we do this.
		 */
		this.userList.html(htmlList).listview('refresh', true);

		//Here is some jQuery magic
		this.userList.find("li").on( "click", this.listItemClickProxy );
	},

	/**
	 * List selection click event listener.
	 *
	 * <P>
	 * We are currently unable to listen for a <code>select</code> event on an
	 * unumbered list in jQuery Mobile so we have to listen for the click on the
	 * whole list and identify the clicked <li /> element.
	 *
	 * @param {Event} event
	 * 		The native Event propagated by jQuery.
	 */
	userList_clickHandler: function( event )
	{
		var li/*HTMLElement*/ = jQuery(event.currentTarget);

		//Index of the clicked list item
		var entry/*Number*/ = parseInt( li.attr("id") );
		jQuery.mobile.changePage("#UserForm");
		this.dispatchEvent( UserList.SELECT, this.users[entry] );
	},

	/**
	 * New button click event listener.
	 */
	newButton_clickHandler: function()
	{
		jQuery.mobile.changePage("#UserForm");
		this.dispatchEvent( UserList.NEW );
	}

});

/*
 * Events type
 */
UserList.NEW		= "new";
UserList.DELETE 	= "delete";
UserList.SELECT 	= "select";